@@ -25,17 +25,12 @@
 
 from collections import defaultdict
 import hashlib
-import os.path
 import re
 from subprocess import DEVNULL, PIPE, Popen
-import tarfile
-from time import gmtime, strftime, strptime
-from urllib.request import urlretrieve
+from time import gmtime, strftime
 
-from rosdistro import get_distribution_cache_string, get_index, get_index_url
 from superflore.exceptions import NoPkgXml
 from superflore.exceptions import UnresolvedDependency
-from superflore.generators.bitbake.oe_query import OpenEmbeddedLayersDB
 from superflore.PackageMetadata import PackageMetadata
 from superflore.utils import err
 from superflore.utils import get_distros
@@ -46,11 +41,17 @@ from superflore.utils import info
 from superflore.utils import make_dir
 from superflore.utils import ok
 from superflore.utils import resolve_dep
-from superflore.utils import warn
 import yaml
 
+UNRESOLVED_PLATFORM_PKG_PREFIX = 'ROS_UNRESOLVED_PLATFORM_PKG_'
+UNRESOLVED_PLATFORM_PKG_REFERENCE_PREFIX = '${'+UNRESOLVED_PLATFORM_PKG_PREFIX
+
 
 class yoctoRecipe(object):
+    """
+    This is used to generate rosdep-resolved.yaml => don't call
+    convert_to_oe_name() on what's added.
+    """
     rosdep_cache = defaultdict(set)
     generated_recipes = dict()
     generated_components = set()
@@ -63,7 +64,7 @@ class yoctoRecipe(object):
 
     def __init__(
         self, component_name, num_pkgs, pkg_name, pkg_xml, distro, src_uri,
-        tar_dir, md5_cache, sha256_cache, skip_keys
+        srcrev_cache, skip_keys
     ):
         self.component = component_name
         yoctoRecipe.max_component_name = max(
@@ -109,28 +110,13 @@ class yoctoRecipe(object):
         self.tdepends = set()
         self.tdepends_external = set()
         self.license_line = None
-        self.archive_name = None
         self.license_md5 = None
-        self.tar_dir = tar_dir
-        if self.getArchiveName() not in md5_cache or \
-           self.getArchiveName() not in sha256_cache:
-            self.downloadArchive()
-            md5_cache[self.getArchiveName()] = hashlib.md5(
-                open(self.getArchiveName(), 'rb').read()).hexdigest()
-            sha256_cache[self.getArchiveName()] = hashlib.sha256(
-                open(self.getArchiveName(), 'rb').read()).hexdigest()
-        self.src_sha256 = sha256_cache[self.getArchiveName()]
-        self.src_md5 = md5_cache[self.getArchiveName()]
+        if self.src_uri not in srcrev_cache:
+            srcrev_cache[self.src_uri] = self.get_srcrev()
+        self.srcrev = srcrev_cache[self.src_uri]
         self.skip_keys = skip_keys
         self.multi_hyphen_re = re.compile('-{2,}')
 
-    def getArchiveName(self):
-        if not self.archive_name:
-            self.archive_name = self.tar_dir + "/" \
-                + self.name.replace('-', '_') + '-' + str(self.version) \
-                + '-' + self.distro + '.tar.gz'
-        return self.archive_name
-
     def get_license_line(self):
         self.license_line = ''
         self.license_md5 = ''
@@ -146,18 +132,70 @@ class yoctoRecipe(object):
                 self.license_md5 = md5.hexdigest()
                 break
 
-    def downloadArchive(self):
-        if os.path.exists(self.getArchiveName()):
-            info("using cached archive for package '%s'..." % self.name)
-        else:
-            info("downloading archive version for package '%s' from %s..." %
-                 (self.name, self.src_uri))
-            urlretrieve(self.src_uri, self.getArchiveName())
+    def get_repo_src_uri(self):
+        """
+        Parse out the git repository SRC_URI out of github archive, e.g.
+        github.com/ros2-gbp/ament_lint-release
+        from
+        https://github.com/ros2-gbp/ament_lint-release/archive/release/bouncy/ament_cmake_copyright/0.5.2-0.tar.gz
+        don't include the protocol, because bitbake git fetcher will use
+        git://...;protocol=https
+        while get_srcrev will need
+        https://...
+        """
+        github_start = 'https://github.com/'
+        structure = self.src_uri.replace(github_start, '')
+        dirs = structure.split('/')
+        return "github.com/%s/%s" % (dirs[0], dirs[1])
 
-    def extractArchive(self):
-        tar = tarfile.open(self.getArchiveName(), "r:gz")
-        tar.extractall()
-        tar.close()
+    def get_repo_branch_name(self):
+        """
+        Parse out the git branch name out of github archive SRC_URI, e.g.
+        release/bouncy/ament_cmake_copyright
+        from
+        https://github.com/ros2-gbp/ament_lint-release/archive/release/bouncy/ament_cmake_copyright/0.5.2-0.tar.gz
+        """
+        github_start = 'https://github.com/'
+        structure = self.src_uri.replace(github_start, '')
+        dirs = structure.split('/')
+        return '{0}/{1}/{2}'.format(
+            dirs[3], dirs[4], dirs[5]).replace('.tar.gz', '')
+
+    def get_repo_tag_name(self):
+        """
+        Parse out the git tag name out of github archive SRC_URI, e.g.
+        release/bouncy/ament_cmake_copyright/0.5.2-0
+        from
+        https://github.com/ros2-gbp/ament_lint-release/archive/release/bouncy/ament_cmake_copyright/0.5.2-0.tar.gz
+        """
+        github_start = 'https://github.com/'
+        structure = self.src_uri.replace(github_start, '')
+        dirs = structure.split('/')
+        return '{0}/{1}/{2}/{3}'.format(
+            dirs[3], dirs[4], dirs[5], dirs[6]).replace('.tar.gz', '')
+
+    def get_srcrev(self):
+        # e.g. git ls-remote https://github.com/ros2-gbp/ament_lint-release \
+        #                    release/bouncy/ament_cmake_copyright/0.5.2-0
+        # 48bf1aa1cb083a884fbc8520ced00523255aeaed \
+        #     refs/tags/release/bouncy/ament_cmake_copyright/0.5.2-0
+        # from https://github.com/ros2-gbp/ament_lint-release/archive/ \
+        #     release/bouncy/ament_cmake_copyright/0.5.2-0.tar.gz
+        from git.cmd import Git
+
+        g = Git()
+        for ref in g.execute(["git",
+                              "ls-remote",
+                              "https://%s" % self.get_repo_src_uri(),
+                              "refs/tags/%s" % self.get_repo_tag_name()
+                              ]).split('\n'):
+            srcrev, tag = ref.split('\t')
+            if tag == "refs/tags/%s" % self.get_repo_tag_name():
+                return srcrev
+        err("Cannot map refs/tags/%s to srcrev in https://%s repository with "
+            "git ls-remote" % (self.get_repo_tag_name(),
+                               self.get_repo_src_uri()))
+        return "INVALID"
 
     def add_build_depend(self, bdepend, internal=True):
         if bdepend not in self.skip_keys:
@@ -213,27 +251,13 @@ class yoctoRecipe(object):
                 if tdepend not in self.tdepends:
                     self.tdepends_external.add(tdepend)
 
-    def get_src_location(self):
-        """
-        Parse out the folder name.
-        TODO(allenh1): add a case for non-GitHub packages,
-        after they are supported.
-        """
-        github_start = 'https://github.com/'
-        structure = self.src_uri.replace(github_start, '')
-        dirs = structure.split('/')
-        return '{0}-{1}-{2}-{3}-{4}'.format(dirs[1], dirs[3],
-                                            dirs[4], dirs[5],
-                                            dirs[6]).replace('.tar.gz', '')
-
     def get_top_inherit_line(self):
-        ret = 'inherit ros_distro_${ROS_DISTRO}\n'
+        ret = 'inherit ros_distro_{0}\n'.format(self.distro)
         ret += 'inherit ros_superflore_generated\n\n'
         return ret
 
     def get_bottom_inherit_line(self):
-        ret = 'inherit ${ROS_COMPONENT_TYPE}_component\n'
-        ret += 'inherit ros_${ROS_BUILD_TYPE}\n'
+        ret = 'inherit ros_${ROS_BUILD_TYPE}\n'
         return ret
 
     def trim_hyphens(self, s):
@@ -245,8 +269,15 @@ class yoctoRecipe(object):
         return self.trim_hyphens(l.translate(conversion_table))
 
     @staticmethod
-    def get_native_suffix(is_native=False):
-        return '-native' if is_native else ''
+    def modify_name_if_native(dep, is_native):
+        """
+        If the name is for an unresolved platform package, move the "-native"
+        inside the "}" so that it's part of the variable name.
+        """
+        if dep.startswith(UNRESOLVED_PLATFORM_PKG_REFERENCE_PREFIX):
+            return dep[0:-len('}')] + ('-native}' if is_native else '}')
+        else:
+            return dep + ('-native' if is_native else '')
 
     @staticmethod
     def get_spacing_prefix():
@@ -286,12 +317,17 @@ class yoctoRecipe(object):
             dep = dep[:-len('_dev')] + '-rosdev'
         elif dep in ('ros1', 'ros2'):
             dep += '--distro-renamed'
-        return cls.convert_dep_except_oe_vars(dep) \
-            + cls.get_native_suffix(is_native)
+        return cls.modify_name_if_native(
+            cls.convert_dep_except_oe_vars(dep),
+            is_native)
 
     @classmethod
     def generate_multiline_variable(cls, var, container, sort=True, key=None):
         if sort:
+            """
+            TODO(herb-kuta-lge): Have default <key> drop trailing '}' so that
+            "${..._foo-native}" sorts after "${..._foo}".
+            """
             container = sorted(container, key=key)
         assignment = '{0} = "'.format(var)
         expression = '"\n'
@@ -322,44 +358,20 @@ class yoctoRecipe(object):
                 for res in results:
                     recipe = self.convert_to_oe_name(res, is_native)
                     dependencies.add(recipe)
-                    system_dependencies.add(self.convert_to_oe_name(res))
+                    system_dependencies.add(recipe)
                     yoctoRecipe.rosdep_cache[dep].add(res)
                     info('External dependency add: ' + recipe)
             except UnresolvedDependency:
-                info('Unresolved dependency: ' + dep)
-                if dep in yoctoRecipe.rosdep_cache:
-                    cached_deps = yoctoRecipe.rosdep_cache[dep]
-                    if cached_deps == set(['null']):
-                        system_dependencies.add(dep)
-                        recipe = dep + self.get_native_suffix(is_native)
-                        dependencies.add(recipe)
-                        msg = 'Failed to resolve (cached):'
-                        warn('{0} {1}: {2}'.format(msg, dep, recipe))
-                    elif cached_deps:
-                        system_dependencies |= cached_deps
-                        for d in cached_deps:
-                            recipe = self.convert_to_oe_name(d, is_native)
-                            dependencies.add(recipe)
-                            msg = 'Resolved in OpenEmbedded (cached):'
-                            info('{0} {1}: {2}'.format(msg, dep, recipe))
-                    continue
-                oe_query = OpenEmbeddedLayersDB()
-                oe_query.query_recipe(self.convert_to_oe_name(dep))
-                if oe_query.exists():
-                    recipe = self.convert_to_oe_name(oe_query.name, is_native)
-                    dependencies.add(recipe)
-                    oe_name = self.convert_to_oe_name(oe_query.name)
-                    system_dependencies.add(oe_name)
-                    yoctoRecipe.rosdep_cache[dep].add(oe_name)
-                    info('Resolved in OpenEmbedded: ' + dep + ' as ' +
-                         oe_query.name + ' in ' + oe_query.layer +
-                         ' as recipe ' + recipe)
-                else:
-                    recipe = dep + self.get_native_suffix(is_native)
-                    dependencies.add(recipe)
-                    system_dependencies.add(dep)
-                    yoctoRecipe.rosdep_cache[dep].add('null')
-                    warn('Failed to resolve fully: ' + dep)
+                unresolved_name = UNRESOLVED_PLATFORM_PKG_REFERENCE_PREFIX\
+                    + dep + '}'
+                recipe = self.convert_to_oe_name(unresolved_name, is_native)
+                dependencies.add(recipe)
+                system_dependencies.add(recipe)
+                # Never add -native.
+                rosdep_name = self.convert_to_oe_name(unresolved_name, False)
+                yoctoRecipe.rosdep_cache[dep].add(rosdep_name)
+                info('Unresolved external dependency add: ' + recipe)
+
         return dependencies, system_dependencies
 
     def get_recipe_text(self, distributor):
@@ -471,46 +483,17 @@ class yoctoRecipe(object):
         ret += '${ROS_BUILDTOOL_EXPORT_DEPENDS}"\n\n'
         ret += 'RDEPENDS_${PN} += "${ROS_EXEC_DEPENDS}"' + '\n\n'
         # SRC_URI
-        ret += 'SRC_URI = "' + self.src_uri + ';'
-        ret += 'downloadfilename=${ROS_SP}.tar.gz"\n'
-        ret += 'SRC_URI[md5sum] = "' + self.src_md5 + '"\n'
-        ret += 'SRC_URI[sha256sum] = "' + self.src_sha256 + '"\n'
-        ret += 'S = "${WORKDIR}/'
-        ret += self.get_src_location() + '"\n\n'
-        ret += 'ROS_COMPONENT_TYPE = "${@ros_distro__get_component_type(\''
-        ret += self.oe_component + '\', d)}"\n'
+        ret += '# matches with: ' + self.src_uri + '\n'
+        ret += 'ROS_BRANCH ?= "branch=' + self.get_repo_branch_name() + '"\n'
+        ret += 'SRC_URI = "git://' + self.get_repo_src_uri() + \
+            ';${ROS_BRANCH};protocol=https"\n'
+        ret += 'SRCREV = "' + self.srcrev + '"\n'
+        ret += 'S = "${WORKDIR}/git"\n\n'
         ret += 'ROS_BUILD_TYPE = "' + self.build_type + '"\n'
-        # include
-        ret += '\n# Allow the above settings to be overridden.\n'
-        ret += 'ROS_INCLUDES_TREE := '
-        ret += '"${@ros_superflore_generated__get_includes_tree(\''
-        ret += self.oe_component + '\', d)}"\n'
-        inc_prefix = 'include ${ROS_LAYERDIR}/'
-        component_path = '/' + self.oe_component + '/' + self.oe_component
-        inc_suffix = '_common.inc\n'
-        ret += inc_prefix + '${ROS_INCLUDES_TREE}' + component_path + \
-            inc_suffix
-        ret += inc_prefix + '${ROS_INCLUDES_TREE}' + \
-            component_path + '-${PV}' + inc_suffix
-        path_prefix = inc_prefix + '${ROS_INCLUDES_TREE}/' + self.oe_component
-        ret += path_prefix + '/${BPN}.inc\n'
-        ret += path_prefix + '/${BPN}-${PV}.inc\n'
         # Inherits
         ret += '\n' + self.get_bottom_inherit_line()
         return ret
 
-    @staticmethod
-    def _get_cache_last_modified_time(distro):
-        try:
-            from urllib.request import urlopen
-        except ImportError:
-            from urllib2 import urlopen
-        dist = get_index(get_index_url()).distributions[distro]
-        conn = urlopen(dist['distribution_cache'])
-        last_modified = conn.headers['last-modified']
-        ts = strptime(last_modified, '%a, %d %b %Y %H:%M:%S %Z')
-        return strftime('%Y%m%d%H%M%S', ts)
-
     @staticmethod
     def _get_ros_version(distro):
         distros = get_distros()
@@ -518,20 +501,54 @@ class yoctoRecipe(object):
             else int(distros[distro]['distribution_type'][len('ros'):])
 
     @staticmethod
-    def generate_rosdistro_conf(
+    def generate_superflore_datetime_inc(basepath, dist, now):
+        datetime_dir = '{0}/meta-ros{1}-{2}/conf/ros-distro/include/{2}/' \
+            'generated/'.format(
+                basepath, yoctoRecipe._get_ros_version(dist), dist)
+        datetime_file_name = 'superflore-datetime.inc'
+        datetime_path = '{}{}'.format(datetime_dir, datetime_file_name)
+        try:
+            make_dir(datetime_dir)
+            with open(datetime_path, 'w') as datetime_file:
+                datetime_file.write('# {}/{}\n'.format(
+                    dist, datetime_file_name))
+                datetime_file.write('# Generated by superflore -- DO NOT EDIT')
+                datetime_file.write(
+                    '\n#\n# Copyright ' + strftime("%Y", gmtime())
+                    + ' Open Source Robotics Foundation\n\n')
+                datetime_file.write(
+                    '\n# The time, in UTC, associated with the last superflore'
+                    + ' run that resulted in a change to the generated files.'
+                    + ' The date portion is\n# used as the third version field'
+                    + ' of ROS_DISTRO_METADATA_VERSION prior to the first'
+                    + ' release of a ROS_DISTRO.\n')
+                datetime_file.write(
+                    'ROS_SUPERFLORE_GENERATION_DATETIME = "{}"\n'.format(now))
+                ok('Wrote {0}'.format(datetime_path))
+        except OSError as e:
+            err('Failed to write SuperFlore datetime {} to disk! {}'.format(
+                datetime_path, e))
+            raise e
+
+    @staticmethod
+    def generate_ros_distro_inc(
             basepath, distro, version, platforms, skip_keys=[]):
-        conf_dir = '{}/conf/ros-distro/include/{}/'.format(basepath, distro)
-        conf_file_name = 'generated-ros-distro.inc'
+        conf_dir = '{0}/meta-ros{1}-{2}/conf/ros-distro/include/{2}/' \
+                    'generated/'.format(
+                        basepath, yoctoRecipe._get_ros_version(distro), distro)
+        conf_file_name = 'superflore-ros-distro.inc'
         conf_path = '{}{}'.format(conf_dir, conf_file_name)
         try:
             make_dir(conf_dir)
             with open(conf_path, 'w') as conf_file:
-                conf_file.write('# {}/{}\n'.format(distro, conf_file_name))
+                conf_file.write('# {}/generated/{}\n'.format(
+                    distro, conf_file_name))
                 conf_file.write('# Generated by superflore -- DO NOT EDIT')
                 conf_file.write(
                     ' (except ROS_DISTRO_METADATA_VERSION_REVISION)\n#\n')
                 conf_file.write(
-                    '# Copyright 2019 Open Source Robotics Foundation\n\n')
+                    '# Copyright ' + strftime("%Y", gmtime())
+                    + ' Open Source Robotics Foundation\n\n')
                 conf_file.write(
                     '# Increment every time meta-ros is released because of '
                     + 'a manually created change, ie, NOT as a result of a '
@@ -541,7 +558,7 @@ class yoctoRecipe(object):
                 conf_file.write(
                     '\nROS_SUPERFLORE_PROGRAM_VERSION = "{}"\n'
                     .format(get_superflore_version()))
-                conf_file.write('ROS_SUPERFLORE_GENERATION_SCHEME = "1"\n')
+                conf_file.write('ROS_SUPERFLORE_GENERATION_SCHEME = "2"\n')
                 ros_version = yoctoRecipe._get_ros_version(distro)
                 conf_file.write(
                     '\nROS_DISTRO_TYPE = "ros{}"\n'.format(ros_version))
@@ -551,16 +568,7 @@ class yoctoRecipe(object):
                 if ros_version == 1:
                     ros_python_version = 2
                 conf_file.write(
-                    'ROS_PYTHON_VERSION = "{}"\n'.format(ros_python_version))
-                conf_file.write(
-                    '\n# The last-modified timestamp in UTC of the upstream '
-                    + 'ROS_DISTRO-cache.yaml from which superflore generated '
-                    + 'the recipes. The date\n# portion is used as the second'
-                    + ' version field of DISTRO_VERSION prior to the first '
-                    + 'release of a ROS_DISTRO.')
-                conf_file.write(
-                    '\nROS_SUPERFLORE_GENERATION_DATETIME = "{0}"\n\n'.format(
-                        yoctoRecipe._get_cache_last_modified_time(distro)))
+                    'ROS_PYTHON_VERSION = "{}"\n\n'.format(ros_python_version))
                 oe_skip_keys = map(
                     lambda skip_key: yoctoRecipe.convert_to_oe_name(skip_key),
                     skip_keys
@@ -579,7 +587,7 @@ class yoctoRecipe(object):
                     '# Number of commits that will be returned by'
                     + ' "git log files/ROS_DISTRO-cache.yaml" when the '
                     + 'generated files are committed. This is\n# used for the'
-                    + ' third version field of DISTRO_VERSION.\n')
+                    + ' fourth version field of DISTRO_VERSION.\n')
                 version = 1 if not version else len(version.splitlines()) + 1
                 conf_file.write(
                     'ROS_NUM_CACHE_YAML_COMMITS = "{}"'.format(version)
@@ -626,8 +634,12 @@ class yoctoRecipe(object):
                     + 'tools.\n')
                 conf_file.write(
                     yoctoRecipe.generate_multiline_variable(
-                        'ROS_SUPERFLORE_GENERATED_BUILDTOOLS',
+                        'ROS_SUPERFLORE_GENERATED_BUILDTOOLS_%s' %
+                        distro.upper(),
                         yoctoRecipe.generated_native_recipes) + '\n')
+                conf_file.write('ROS_SUPERFLORE_GENERATED_BUILDTOOLS_append ='
+                                ' " ${ROS_SUPERFLORE_GENERATED_BUILDTOOLS_%s}"'
+                                '\n\n' % distro.upper())
                 conf_file.write(yoctoRecipe.generate_multiline_variable(
                     'ROS_SUPERFLORE_GENERATED_PLATFORM_PACKAGE_DEPENDENCIES',
                     yoctoRecipe.platform_deps))
@@ -643,61 +655,37 @@ class yoctoRecipe(object):
                     yoctoRecipe.generate_multiline_variable(
                         'ROS_SUPERFLORE_GENERATED_RECIPES_FOR_COMPONENTS',
                         yoctoRecipe.generated_components))
+                conf_file.write(
+                    '\n# Platform packages without a OE-RECIPE@OE-LAYER'
+                    + ' mapping in base.yaml, python.yaml, or ruby.yaml. Until'
+                    + ' they are added, override\n# the settings in'
+                    + ' ros-distro.inc .\n')
+                """
+                Drop trailing "}" so that "..._foo-native" sorts after
+                "..._foo".
+                """
+                unresolved = [p[0:-1] for p in yoctoRecipe.platform_deps
+                              if p.startswith(
+                                UNRESOLVED_PLATFORM_PKG_REFERENCE_PREFIX)]
+                for p in sorted(unresolved):
+                    """
+                    PN is last underscore-separated field. NB the trailing '}'
+                    has already been removed.
+                    """
+                    pn = p.split('_')[-1]
+                    conf_file.write(
+                        UNRESOLVED_PLATFORM_PKG_PREFIX + pn + ' = "UNRESOLVED-'
+                        + pn + '"\n')
+
                 ok('Wrote {0}'.format(conf_path))
         except OSError as e:
             err('Failed to write conf {} to disk! {}'.format(conf_path, e))
             raise e
 
-    @staticmethod
-    def generate_distro_cache(basepath, distro, skip_keys=[]):
-        distro_cache_dir = '{0}/files/{1}/'.format(basepath, distro)
-        distro_cache_path = '{0}cache.yaml'.format(distro_cache_dir)
-        try:
-            index = get_index(get_index_url())
-            yaml_str = get_distribution_cache_string(index, distro)
-            make_dir(distro_cache_dir)
-            with open(distro_cache_path, 'w') as distro_cache_file:
-                distro_cache_file.write('# {}/cache.yaml\n'.format(distro))
-                distro_cache_file.write(yaml_str)
-                ok('Wrote {0}'.format(distro_cache_path))
-        except OSError as e:
-            err('Failed to write distro cache {} to disk! {}'.format(
-                distro_cache_path, e))
-            raise e
-        # Generate a diff'able cache file
-        distro_cache_diff_path = '{}cache.diffme'.format(distro_cache_dir)
-        try:
-            def replace_all_patterns(d, text):
-                for k, v in d.items():
-                    text = re.sub(k, v, text, flags=re.M)
-                return text
-            replacement_table = {
-                r"{([^ }][^ }]*)}": r'[[\1]]',
-                r"{": r"{\n",
-                r"}": r"\n}",
-                r"\[\[": r"{",
-                r"\]\]": r"}",
-                r", ": r",\n",
-                r"^    ": r"-----\n",
-                r"<version>[^<]*</version>": r"",
-                r"><": r">\n<",
-                r"^  ": r"-----\n",
-                r"^(source_repo_package_xmls:)": r"-----\n\1",
-            }
-            with open(distro_cache_diff_path, 'w') as distro_cache_diff_file:
-                distro_cache_diff_file.write(
-                    '# {}/cache.diffme\n'.format(distro))
-                yaml_str = replace_all_patterns(replacement_table, yaml_str)
-                distro_cache_diff_file.write(yaml_str)
-                ok('Wrote {0}'.format(distro_cache_diff_path))
-        except OSError as e:
-            err('Failed to write diffme distro cache {} to disk! {}'.format(
-                distro_cache_diff_path, e))
-            raise e
-
     @staticmethod
     def generate_rosdep_resolve(basepath, distro):
-        rosdep_resolve_dir = '{0}/files/{1}/'.format(basepath, distro)
+        rosdep_resolve_dir = '{0}/meta-ros{1}-{2}/files/{2}/generated/'.format(
+            basepath, yoctoRecipe._get_ros_version(distro), distro)
         rosdep_resolve_path = '{0}rosdep-resolve.yaml'.format(
             rosdep_resolve_dir)
         try:
@@ -717,14 +705,16 @@ class yoctoRecipe(object):
 
     @staticmethod
     def generate_superflore_change_summary(basepath, distro, change_summary):
-        change_summary_dir = '{0}/files/{1}/'.format(basepath, distro)
+        change_summary_dir = '{0}/meta-ros{1}-{2}/files/{2}/generated/'.format(
+            basepath, yoctoRecipe._get_ros_version(distro), distro)
         change_summary_path = '{0}superflore-change-summary.txt'.format(
             change_summary_dir)
         try:
             make_dir(change_summary_dir)
             with open(change_summary_path, 'w') as change_summary_file:
                 change_summary_file.write(
-                    '{}/superflore-change-summary.txt\n'.format(distro))
+                    '{}/generated/superflore-change-summary.txt\n'.format(
+                        distro))
                 change_summary_file.write(change_summary)
                 ok('Wrote {0}'.format(change_summary_path))
         except OSError as e:
@@ -734,7 +724,10 @@ class yoctoRecipe(object):
 
     @staticmethod
     def generate_newer_platform_components(basepath, distro):
-        newer_sys_comps_dir = '{0}/files/{1}/'.format(basepath, distro)
+        newer_sys_comps_dir = '{0}/meta-ros{1}-{2}/files/{2}/' \
+                              'generated/'.format(
+                                  basepath,
+                                  yoctoRecipe._get_ros_version(distro), distro)
         newer_sys_comps_path = '{0}newer-platform-components.list'.format(
             newer_sys_comps_dir)
         ros_version = yoctoRecipe._get_ros_version(distro)
